%
% allgemeineFunktionalitaeten.pl
%
% Hier werden allgemeine, oefter verwendete Klauseln 
% verwaltet.
%

% letzte Aenderung 4.2.02 rs: erweitern von erstelleTagRaumListe
% fuer den Umgang mit Listen

% letzte Aenderung 6.2.02 rs: einfuegen der Hilfspraedikate von 
% Matthias aus allgPraedikate.pl

/***********************************************************/
% einbinden der Bibliothek fuer Listen
/***********************************************************/
:- use_module(library(lists)).

/***********************************************************/
/***********************************************************/

/***********************************************************/
/* dozent(+X)                                              */
/***********************************************************/
/* wahr, wenn ein Element (atom) auch gleichzeitig ein     */
/* Dozent ist                                              */
/***********************************************************/
dozent(X) :- 
	dozenten(Dozenten), 
	member(X, Dozenten).

/***********************************************************/
/* erstelleTagListe(-TagListe)                             */
/***********************************************************/
/* Heraussuchen aller Tage                                 */
/***********************************************************/
erstelleTagListe(TagListe) :-
        findall(Tag, tag(Tag, _), TagListe).

/***********************************************************/
/* erstelleZeitListe(-ZeitListe)                           */
/***********************************************************/
/* Heraussuchen aller Zeiten                               */
/***********************************************************/
erstelleZeitListe(ZeitListe) :-
        findall(Zeit, zeit(Zeit, _), ZeitListe).

/***********************************************************/
/* erstelleRaumListe(-RaumListe)                           */
/***********************************************************/
/* Heraussuchen aller Raeume                               */
/***********************************************************/
erstelleRaumListe(RaumListe) :-
        findall(Raum, raum(Raum, _, _, _), RaumListe).

/***********************************************************/
/* erstelleTagRaumListe(+TagListe, +RaumListe,             */
/*   +ZeitListe, -TagRaumListe)                            */
/***********************************************************/
/* Erstellen einer nach Tag-Raum-Zeit sortierten Liste,    */
/* die fuer jede Zeiteinheit, jeden Raum und jeden Tag ein */
/* Element (Veranstaltung bzw. leere Liste) enthaelt       */
/***********************************************************/
erstelleTagRaumListe([_], [_], [], []).
erstelleTagRaumListe([_|TagListe], [_], [], TagRaumListe) :-
        erstelleZeitListe(ZeitListe),
        erstelleRaumListe(RaumListe),
        erstelleTagRaumListe(TagListe, RaumListe, ZeitListe, TagRaumListe).
erstelleTagRaumListe(TagListe, [_|RaumListe], [], TagRaumListe) :-
        erstelleZeitListe(ZeitListe),
        erstelleTagRaumListe(TagListe, RaumListe, ZeitListe, TagRaumListe).
erstelleTagRaumListe([TagKopf|TagListe], [RaumKopf|RaumListe], [ZeitKopf|ZeitListe], [TagRaumKopf|TagRaumRest]) :-
        findall((Name, Teilnehmer, TagKopf, RaumKopf, ZeitKopf), (veranstaltungAmUmIn(Name, [TagKopf], [ZeitKopf], RaumKopf), veranstaltung(Name, Teilnehmer, _)), TagRaumKopf),
        erstelleTagRaumListe([TagKopf|TagListe], [RaumKopf|RaumListe], ZeitListe, TagRaumRest).

/***********************************************************/
/***********************************************************/

/***********************************************************/
% heraussuchen besonderer Veranstaltungsarten
/***********************************************************/

/***********************************************************/
/* findeDoppelstunden(-Ergebnis)                           */
/***********************************************************/
findeDoppelstunden(Ergebnis) :-
	findall(Name1,  doppelstunde(Name1, _), DoppelstundenListe1),
	findall(Name2,  doppelstunde(_, Name2), DoppelstundenListe2),
	append(DoppelstundenListe1, DoppelstundenListe2, Ergebnis).

/***********************************************************/
/* findeDreifachstunden(-Ergebnis)                         */
/***********************************************************/
findeDreifachstunden(Ergebnis) :-
	findall(Name1, dreifachstunde(Name1, _, _), DreifachstundenListe1),
	findall(Name2, dreifachstunde(_, Name2, _), DreifachstundenListe2),
	findall(Name3, dreifachstunde(_, _, Name3), DreifachstundenListe3),
	append(DreifachstundenListe1, DreifachstundenListe2, HilfsListe), 
	append(HilfsListe, DreifachstundenListe3, Ergebnis).

/***********************************************************/
/* findeVierfachstunden(-Ergebnis)                    
/***********************************************************/
findeVierfachstunden(Ergebnis) :-
	findall(Name1, vierfachstunde(Name1, _, _, _), VierfachstundenListe1),
	findall(Name2, vierfachstunde(_, Name2, _, _), VierfachstundenListe2),
	findall(Name3, vierfachstunde(_, _, Name3, _), VierfachstundenListe3),
	findall(Name4, vierfachstunde(_, _, _, Name4), VierfachstundenListe4),
	append(VierfachstundenListe1, VierfachstundenListe2, HilfsListe1),
	append(HilfsListe1, VierfachstundenListe3, HilfsListe2),
	append(HilfsListe2, VierfachstundenListe4, Ergebnis).

/***********************************************************/
/* findeVeranstaltungenAmSelbenTagOhneWuensche(-Ergebnis)  */
/***********************************************************/
findeVeranstaltungenAmSelbenTagOhneWuensche(Ergebnis):-
	findall(Name, veranstaltungenAmSelbenTag(Name, _), Liste1),
	findall(Name, veranstaltungenAmSelbenTag(_, Name), Liste2),
	findall(Name, (veranstaltungenAmSelbenTag(Name, _, X), not(number(X))), Liste3),
	findall(Name, (veranstaltungenAmSelbenTag(_, Name, X), not(number(X))), Liste4),
	findall(Name, (veranstaltungenAmSelbenTag(_, _, Name), not(number(Name))), Liste5),
	findall(Name, (veranstaltungenAmSelbenTag(Name, _, _, X), not(number(X))), Liste6),
	findall(Name, (veranstaltungenAmSelbenTag(_, Name, _, X), not(number(X))), Liste7),
	findall(Name, (veranstaltungenAmSelbenTag(_, _, Name, X), not(number(X))), Liste8),
	findall(Name, (veranstaltungenAmSelbenTag(_, _, _, Name), not(number(Name))), Liste9),
	append(Liste1, Liste2, HilfsListe1),
	append(HilfsListe1, Liste3, HilfsListe2),
	append(HilfsListe2, Liste4, HilfsListe3),
	append(HilfsListe3, Liste5, HilfsListe4),
	append(HilfsListe4, Liste6, HilfsListe5),
	append(HilfsListe5, Liste7, HilfsListe6),
	append(HilfsListe6, Liste8, HilfsListe7),
	append(HilfsListe7, Liste9, Ergebnis).

/***********************************************************/
/* findeVeranstaltungenNichtAmSelbenTagOhneWuensche(       */
/*	-Ergebnis)                                         */
/***********************************************************/
findeVeranstaltungenNichtAmSelbenTagOhneWuensche(Ergebnis) :-	
	findall(Name, veranstaltungenNichtAmSelbenTag(Name, _), Liste1),
	findall(Name, veranstaltungenNichtAmSelbenTag(_, Name), Liste2),
	findall(Name, (veranstaltungenNichtAmSelbenTag(Name, _, X), not(number(X))), Liste3),
	findall(Name, (veranstaltungenNichtAmSelbenTag(_, Name, _), not(number(X))), Liste4),
	findall(Name, (veranstaltungenNichtAmSelbenTag(_, _, Name), not(number(Name))), Liste5),
	findall(Name, (veranstaltungenNichtAmSelbenTag(Name, _, _, X), not(number(X))), Liste6),
	findall(Name, (veranstaltungenNichtAmSelbenTag(_, Name, _, X), not(number(X))), Liste7),
	findall(Name, (veranstaltungenNichtAmSelbenTag(_, _, Name, X), not(number(X))), Liste8),
	findall(Name, (veranstaltungenNichtAmSelbenTag(_, _, _, Name), not(number(Name))), Liste9),
	append(Liste1, Liste2, HilfsListe1),
	append(HilfsListe1, Liste3, HilfsListe2),
	append(HilfsListe2, Liste4, HilfsListe3),
	append(HilfsListe3, Liste5, HilfsListe4),
	append(HilfsListe4, Liste6, HilfsListe5),
	append(HilfsListe5, Liste7, HilfsListe6),
	append(HilfsListe6, Liste8, HilfsListe7),
	append(HilfsListe7, Liste9, Ergebnis).


/***********************************************************/
/* wandleVeranstaltungenInZahlen(+Veranstaltungen,         */
/*	-Ergebnis)                                         */
/***********************************************************/
/* Wandeln von Name -> Nummer                              */
/***********************************************************/
wandleVeranstaltungenInZahlen([], []).
wandleVeranstaltungenInZahlen([Kopf|Rest], [ErgebnisKopf|ErgebnisRest]) :-
	veranstaltung(ID, Kopf, _, _, _),
	append([0, ID], [], ErgebnisKopf),
	wandleVeranstaltungenInZahlen(Rest, ErgebnisRest).

/***********************************************************/
/***********************************************************/

/***********************************************************/
% Listenfunktionalitaeten, die in ECLiPSe nicht
% definiert sind ...
/***********************************************************/

/*******************************************************/
/* nth/3                                               */
/* nth(+Ele, +Liste, -Pos)                             */
/*******************************************************/
/* an welcher Pos in einer Liste steht ein Ele         */
/*******************************************************/
nth(_, [], 0) :- 
  !.
nth(Element, Liste, 0) :-
  nonmember(Element, Liste),
  !.
nth(Element, [Element|_], 1) :- 
  !.
nth(Element, [_|Rest], Erg) :-
  nth(Element, Rest, NeuErg),
  Erg is NeuErg + 1.

/*******************************************************/
/* getnth/3                                            */
/* getnth(-Ele, +Liste, +Pos)                          */
/*******************************************************/
/* welches Ele in einer Liste steht an der Stelle Pos  */
/*******************************************************/
getnth(_, _, 0) :- !, fail.
getnth(_, [], _) :- !, fail.
getnth(_, Liste, Pos) :-
  Liste = [_|_],
  length(Liste, N),
  Pos > N,
  !,
  fail.
getnth(Kopf, [Kopf|_], 1) :-
  !.
getnth(Ele, [_|Rest], Pos) :-
  NPos is Pos - 1,
  getnth(Ele, Rest, NPos).

/*******************************************************/
/* last/2                                              */
/*******************************************************/
/* last(+Liste, Element)                               */
/* holt das letzte Element einer Liste                 */
/*******************************************************/
last([], _) :- fail.
last(Liste, Element) :-
  reverse(Liste, [Element|_]).

/***********************************************************/
/* letztes(Wert, +Liste)                                  */
/***********************************************************/
/* Bestimmen des letzten Elements einer Liste              */
/***********************************************************/
letztes(LetztesElement, [LetztesElement]).
letztes(LetztesElement, [_|Rest]) :- 
	letztes(LetztesElement, Rest).

/*******************************************************/
/* multimember/3                                       */
/*******************************************************/
/* multimember(Listen, Element, Anzahl)                */
/* berechnet in wieviel Listen das Element Member ist  */
/*******************************************************/
multimember([], _, 0) :-
  !.
multimember([Kopf|Rest], Element, Anz) :-
  memberchk(Element, Kopf),
  multimember(Rest, Element, NAnz),
  !,
  Anz is NAnz + 1.
multimember([Kopf|Rest], Element, Anz) :-
  nonmember(Element, Kopf),
  multimember(Rest, Element, Anz).

/***********************************************************/
/* findeDoppelte(+Liste, -Ergebnis)                        */
/***********************************************************/
/* Testen, ob in einer Liste doppelte Elemente vorkommen.  */
/***********************************************************/
findeDoppelte([], []).
findeDoppelte(Liste, []) :-
	length(Liste, N),
	sort(Liste, ListeSort),
	length(ListeSort, M),
	N == M.
findeDoppelte([Kopf|Rest], Ergebnis) :-
	nonmember(Kopf, Rest),
	findeDoppelte(Rest, Ergebnis).
findeDoppelte([Kopf|Rest], [ErgebnisKopf|ErgebnisRest]) :-
	member(Kopf, Rest),
	append([Kopf], [], ErgebnisKopf),
	findeDoppelte(Rest, ErgebnisRest).

/*******************************************************/
/* firstNElems/4                                       */
/* firstNElems(Liste, N, FirstNElems, Rest)            */
/*******************************************************/
/* teilt Liste in 2 Teillisten nach dem ntem Element   */
/*******************************************************/
firstNElems([], _, [], []) :- 
   !.
firstNElems(Liste, N, [], Liste) :-
   N < 1, 
   !.
firstNElems(Liste, N, Liste, []) :-
   length(Liste, Laenge),
   N >= Laenge, 
   !.
firstNElems([Kopf|Rest1], N, [Kopf|FirstNElems], Rest3) :-
   NeuN is N - 1,
   firstNElems(Rest1, NeuN, FirstNElems, Rest3).

/*******************************************************/
/* sortAbsteigend/2                                    */
/* sortAbsteigend(Liste, AbstSortListe)                */
/*******************************************************/
/* sortiert Liste in absteigender Reihenfolge          */
/*******************************************************/
sortAbsteigend(Liste, AbstSortListe) :-
   sort(Liste, SortListe),
   reverse(SortListe, AbstSortListe).

/*******************************************************/
/* listenElementeFlatten/2                             */
/* listenElementeFlatten(NotFlatErgebnis, Ergebnis)    */
/*******************************************************/
/* Flachmachen aller eingenisteten Listen innerhalb    */
/* eines Listenelementes.                              */
/*******************************************************/
listenElementeFlatten([], []).
listenElementeFlatten([Kopf|Rest], [ErgebnisKopf|ErgebnisRest]) :-
  flatten(Kopf, ErgebnisKopf),
  listenElementeFlatten(Rest, ErgebnisRest).

/*********************************************************/
/* listeBauen/3                                          */
/* listeBauen(Laenge, Element, Liste)                    */
/*******************************************************/
/* baut eine Liste                                       */
/*********************************************************/
listeBauen(0, _, []).
listeBauen(Laenge, Element, [Element|Rest]) :-
  NeuLaenge is Laenge - 1,
  listeBauen(NeuLaenge, Element, Rest). 

/*******************************************************/
/* listeBauen2/2                                       */
/* listeBauen2(Laenge, Liste)                          */
/*******************************************************/
/* baut eine Liste                                     */
/*******************************************************/
listeBauen2(0, []).
listeBauen2(Laenge, [Laenge|Rest]) :-
  NeuLaenge is Laenge - 1,
  listeBauen2(NeuLaenge, Rest). 

/*******************************************************/
/* anzVorkommen/3                                      */
/* anzVorkommen(Element, Liste, AnzVorkommen)          */
/*******************************************************/
/* berechnet, wie oft ein Element in einer Liste       */
/* vorkommt                                            */
/*******************************************************/
anzVorkommen(Element, Liste, 0) :-
   nonmember(Element, Liste),
   !.
anzVorkommen(_, [], 0) :-
   !.
anzVorkommen(Element, [Kopf|Rest], Anz) :-
   Element = Kopf,
   !,
   anzVorkommen(Element, Rest, NeuAnz),
   Anz is 1 + NeuAnz.
anzVorkommen(Element, [Kopf|Rest], Anz) :-
   not(Element = Kopf),
   !,
   anzVorkommen(Element, Rest, Anz).

/***********************************************************/
/* istTeilmenge(+Liste1, +Liste2)                          */
/* Bestimmen, ob Liste1 eine IstTeilmenge der Liste2 ist   */
/***********************************************************/
istTeilmenge([], _).
istTeilmenge([Kopf1|Rest1], Menge2) :- 
	member(Kopf1, Menge2),
	istTeilmenge(Rest1, Menge2).

/***********************************************************/
/***********************************************************/

/***********************************************************/
% Funktionalitaeten, um herauszufinden, wieviel Spielraum 
% es gibt, die Veranstaltungen eines Teilnehmers zu 
% platzieren
/***********************************************************/

/***********************************************************/
/* findeMehrfachAufgefuehrte(+TeilnehmerListe,             */
/*     -Ergebnis)                                          */
/***********************************************************/
/* Bestimmen der Teilnehmer, die in der Liste mehrfach     */
/* gefuehrt werden, um weitere Berechnungen vorzubereiten. */
/***********************************************************/
findeMehrfachAufgefuehrte([], []).
findeMehrfachAufgefuehrte([TeilnehmerKopf|TeilnehmerRest], MehrfachListe) :-
	findall((TeilnehmerKopf, Am, Um), teilnehmerNichtAmUm(TeilnehmerKopf, Am, Um), Liste),
	length(Liste, 1),
	findeMehrfachAufgefuehrte(TeilnehmerRest, MehrfachListe).
findeMehrfachAufgefuehrte([TeilnehmerKopf|TeilnehmerRest], MehrfachListe) :-
	findall((TeilnehmerKopf, Am, Um), (teilnehmerNichtAmUm(TeilnehmerKopf, Am, Um), (Um = []; Am = [])), Liste),
	length(Liste, 1),
	findeMehrfachAufgefuehrte(TeilnehmerRest, MehrfachListe).
findeMehrfachAufgefuehrte([TeilnehmerKopf|TeilnehmerRest], [Kopf|Rest]) :-
	findall((TeilnehmerKopf, Am, Um), (teilnehmerNichtAmUm(TeilnehmerKopf, Am, Um), (Um = []; Am = [])), Liste),
	append(Liste, [], Kopf),
	findeMehrfachAufgefuehrte(TeilnehmerRest, Rest).

/***********************************************************/
/* erweitereListeUmTage(+TeilnehmerListe, +AlleTage,       */
/*     -Ergebnis)                                          */
/***********************************************************/
/* Einfuegen der Tage, die bei teilnehmerNichtAmUm nicht   */
/* aufgefuehrt werden.                                     */
/***********************************************************/
erweitereListeUmTage([], _, []).
erweitereListeUmTage([Kopf|Rest], TagListe, [ErgebnisKopf|ErgebnisRest]) :-
	ergaenzeTage(Kopf, TagListe, ErgebnisKopf),
	erweitereListeUmTage(Rest, TagListe, ErgebnisRest).

/***********************************************************/
/* ergaenzeTage(+TeilnehmerListe, +TagListe, -Ergebnis)    */
/***********************************************************/
/* Hinzufuegen der Tage.                                   */
/***********************************************************/
ergaenzeTage([], _, _).
ergaenzeTage([(_, [], _)|Rest], TagListe, Ergebnis) :-
	ergaenzeTage(Rest, TagListe, Ergebnis).
ergaenzeTage([(_, Am, _)|Rest], TagListe, Ergebnis) :-
	loescheTage(Am, TagListe, NeueTagListe),
	flatten(NeueTagListe, TagListeFlat),
	subtract(TagListe, TagListeFlat, Ergebnis),
	ergaenzeTage(Rest, NeueTagListe, Ergebnis).

/***********************************************************/
/* loescheTage(+TeilnehmerListe, +TagListe, -Ergebnis)     */
/***********************************************************/
/* Entfernen der Tage.                                     */
/***********************************************************/
loescheTage([], _, []).
loescheTage([Kopf|Rest], TagListe, [TagKopf|TagRest]) :-
	append([Kopf], [], TagKopf),
	loescheTage(Rest, TagListe, TagRest).

/***********************************************************/
/* schreibeTage(+TeilnehmerListe, +TagListe, -Ergebnis)    */
/***********************************************************/
/* Schreiben der Tage.                                     */
/***********************************************************/
schreibeTage([], [], []).
schreibeTage([Kopf1|Rest1], [Kopf2|Rest2], [ErgebnisKopf|ErgebnisRest]) :-
	schreibeTage1(Kopf1, Kopf2, ErgebnisKopf),
	schreibeTage(Rest1, Rest2, ErgebnisRest).

/***********************************************************/
/* schreibeTage1(+TeilnehmerListe, +TagListe,              */
/*     -Ergebnis)                                          */
/***********************************************************/
/* Eigentliches Schreiben der Tage.                        */
/***********************************************************/
schreibeTage1([], _, []).
schreibeTage1([(Teilnehmer, [], Um)|Rest], NeueTage, [ErgebnisKopf|ErgebnisRest]) :-
	append([(Teilnehmer, NeueTage, Um)], [], ErgebnisKopf),
	schreibeTage1(Rest, NeueTage, ErgebnisRest).
schreibeTage1([(Teilnehmer, Am, Um)|Rest], NeueTage, [ErgebnisKopf|ErgebnisRest]) :-
	append([(Teilnehmer, Am, Um)], [], ErgebnisKopf),
	schreibeTage1(Rest, NeueTage, ErgebnisRest).

/***********************************************************/
/* berechneZurVerfuegungStehendeZeiteinheiten(             */
/*     +TeilnehmerListe, +Tage, +Zeiten , -Ergebnis)       */
/***********************************************************/
/* Berechnen der Zeiteinheiten, die fuer einen Teilnehmer  */
/* effektiv zur Verfuegung stehen.                         */
/***********************************************************/
berechneZurVerfuegungStehendeZeiteinheiten([], _, _, []).
berechneZurVerfuegungStehendeZeiteinheiten([(Name, Am, [])|TeilnehmerRest], Tage, Zeiten, [ErgebnisKopf|ErgebnisRest]) :-
	length(Am, AnzTage),
	Zeiteinheiten is AnzTage * Zeiten,
	append([(Name, Zeiteinheiten)], [], ErgebnisKopf),
	berechneZurVerfuegungStehendeZeiteinheiten(TeilnehmerRest, Tage, Zeiten, ErgebnisRest).
berechneZurVerfuegungStehendeZeiteinheiten([(Name, [], Um)|TeilnehmerRest], Tage, Zeiten, [ErgebnisKopf|ErgebnisRest]) :-
	length(Um, AnzZeiten),
	Zeiteinheiten is Tage * AnzZeiten,
	append([(Name, Zeiteinheiten)], [], ErgebnisKopf),
	berechneZurVerfuegungStehendeZeiteinheiten(TeilnehmerRest, Tage, Zeiten, ErgebnisRest).
berechneZurVerfuegungStehendeZeiteinheiten([(Name, Am, Um)|TeilnehmerRest], Tage, Zeiten, [ErgebnisKopf|ErgebnisRest]) :-
	length(Am, AnzTage),
	length(Um, AnzZeiten),
	Zeiteinheiten is AnzTage * AnzZeiten,
	append([(Name, Zeiteinheiten)], [], ErgebnisKopf),
	berechneZurVerfuegungStehendeZeiteinheiten(TeilnehmerRest, Tage, Zeiten, ErgebnisRest).

/***********************************************************/
/* fuehreErgebnisseZusammen(+TeilnehmerListe, -Ergebnis)   */
/***********************************************************/
/* Zusammenfuehren der Ergebnisse, so dass nur noch eine   */
/* Zahl pro Teilnehmer vorhanden ist.                      */
/***********************************************************/
fuehreErgebnisseZusammen([], []).
fuehreErgebnisseZusammen([(Teilnehmer, Einheiten)|Rest], [ErgebnisKopf|ErgebnisRest]) :-
	nonmember((Teilnehmer, _), Rest),
	append([(Teilnehmer, Einheiten)], [], ErgebnisKopf),
	fuehreErgebnisseZusammen(Rest, ErgebnisRest).
fuehreErgebnisseZusammen([(Teilnehmer, Einheiten)|Rest], [ErgebnisKopf|ErgebnisRest]) :-
	fuehreErgebnisseZusammen1([(Teilnehmer, Einheiten)|Rest], Ergebnis, NeueTeilnehmerListe),
	append([(Teilnehmer, Ergebnis)], [], ErgebnisKopf),
	fuehreErgebnisseZusammen(NeueTeilnehmerListe, ErgebnisRest).

/***********************************************************/
/* fuehreErgebnisseZusammen1(+TeilnehmerListe,             */
/*     -Ergebnis)                                          */
/***********************************************************/
/* Eigentliches Zusammenfuehren, wenn Teilnehmer mehrfach  */
/* in der Liste vorhanden ist.                             */
/***********************************************************/
fuehreErgebnisseZusammen1([(Teilnehmer, Einheiten)|Rest], Ergebnis, Liste) :-
	member((Teilnehmer, _), Rest),        
	ErgebnisAkt is Einheiten,
	fuehreErgebnisseZusammen1(Rest, ErgebnisRest, Liste),
	Ergebnis is ErgebnisAkt + ErgebnisRest.
fuehreErgebnisseZusammen1([(Teilnehmer, Einheiten)|Rest], Ergebnis, Liste) :-
	nonmember((Teilnehmer, _), Rest),        
	Ergebnis is Einheiten,
	delete((Teilnehmer, Einheiten), [(Teilnehmer, Einheiten)|Rest], Liste).

/***********************************************************/
/* berechneVeranstaltungenDerTeilnehmer(+TeilnehmerListe,  */
/*     +Max, -Ergebnis)                                    */
/***********************************************************/
/* Herausfinden, wie viele Veranstaltungsverpflichtungen   */
/* ein Teilnehmer besitzt.                                 */
/***********************************************************/
berechneVeranstaltungenDerTeilnehmer([], _, []).
berechneVeranstaltungenDerTeilnehmer([(Name, Frei)|Rest], Max, Ergebnis) :-
	findall(Veranstaltungen, (veranstaltung(Veranstaltungen, Teilnehmer, _), 
	member(Name, Teilnehmer)), VeranstaltungsListe),
	length(VeranstaltungsListe, N),
	Frei + N =< Max,
	berechneVeranstaltungenDerTeilnehmer(Rest, Max, Ergebnis).
berechneVeranstaltungenDerTeilnehmer([(Name, Frei)|Rest], Max, [ErgebnisKopf|ErgebnisRest]) :-
	findall(Veranstaltungen, (veranstaltung(Veranstaltungen, Teilnehmer, _), member(Name, Teilnehmer)), VeranstaltungsListe),
	length(VeranstaltungsListe, N),
	Frei + N > Max,
	append([Name], [], ErgebnisKopf),
	berechneVeranstaltungenDerTeilnehmer(Rest, Max, ErgebnisRest).
