%
% ausgabeXML.pl
%
% Ausgabe eines fertigen Plans in XML
%

% letzte Aenderung 30.11.01 rs: Einfuegen eines Raum-Ende-
% Tags am Ende eines Tages

% letzte Aenderung 25.01.02 rs: Verlagern der Untergruppenersetzung
% in diesen Teil

allgemeineFunktionalitaeten('allgemeineFunktionalitaeten.pl').

/***********************************************************/
/* ausgabeXML(+Datei).                                     */
/***********************************************************/
/* Ausgeben des fertigen Plans in XML.                     */
/***********************************************************/
ausgabeXML('Plan.xml').

/***********************************************************/
/* schreibeXML(+Stammdaten)                                */
/***********************************************************/
/* Steuern des Ablaufs beim Schreiben der XML-Ausgabe      */
/***********************************************************/
schreibeXML(Stammdaten) :-
	allgemeineFunktionalitaeten(AllgemeineFunktionalitaeten),
	compile(AllgemeineFunktionalitaeten),
	writeln("Erzeugen des XML-Outputs ..."),
	writeln("Einlesen der Stammdaten ..."),
	compile(Stammdaten),
	writeln("Einlesen der berechneten Lsung ..."),
	ausgabeDatei(Loesung),
	compile(Loesung),
	writeln("Sortieren der Lsung ..."),
	findall((Untergruppe, Obergruppe), untergruppe(Untergruppe, _, Obergruppe), UGruppenListe),
	findall(veranstaltung(VerName, VerTeilnehmer, VerRessourcen), veranstaltung(_, VerName, VerTeilnehmer, VerRessourcen, _), VeranstaltungsListe),
	ersetzeUntergruppen(VeranstaltungsListe, UGruppenListe, VeranstListeNeu),	
	flatten(VeranstListeNeu, VeranstaltungsListeFlat),
	schreibeVeranstListeOhneUntergruppen(VeranstaltungsListeFlat, 'ZwErgebnis.pl'),
	compile('ZwErgebnis.pl'),
	erstelleTagListe(TagListe),
	erstelleZeitListe(ZeitListe),
	erstelleRaumListe(RaumListe),
	% leere Veranstaltungen einfuegen
	% wenn ein Raum zu einer Zeiteinheit
	% nicht belegt ist.
	erstelleTagRaumListe(TagListe, RaumListe, ZeitListe, TagRaumListe),
	writeln(TagRaumListe),
	writeln("Schreiben der XMLDatei ..."),
	ausgabeXML(XMLDatei),
	initXMLDatei(XMLDatei),
	writeln("    Schreiben des Plans ..."),
	schreibePlanXML(XMLDatei, TagListe, RaumListe, TagRaumListe, ZeitListe),
	schliesseXMLDatei(XMLDatei),
	writeln("Heureka!").

/***********************************************************/
/* initXMLDatei(?ZielDatei)                                */
/***********************************************************/
/* Initialisierung der Ausgabedatei                        */
/***********************************************************/
initXMLDatei(Ziel) :-
        open(Ziel, write, Stream),
        print(Stream, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"),
        nl(Stream),
        print(Stream, '<stundenplan>'),
        nl(Stream),
        close(Stream).

/***********************************************************/
/* schliesseXMLDatei(?ZielDatei)                           */
/***********************************************************/
/* Schliessen der Ausgabedatei                             */
/***********************************************************/
schliesseXMLDatei(Ziel) :-
        open(Ziel, append, Stream),
        nl(Stream),
        print(Stream, '</stundenplan>'),
        nl(Stream),
        close(Stream).

/***********************************************************/
/* schreibePlanXML(?ZielDatei, +TagListe, +RaumListe,      */
/*      +TagRaumListe, +ZeitListe)                         */
/***********************************************************/
/* Ablaufsteuerung des Planschreibens                      */
/***********************************************************/
schreibePlanXML(Ziel, TagListe, RaumListe, TagRaumListe, ZeitListe) :-
	open(Ziel, append, Stream),
	schreibePlanXML1(TagListe, RaumListe, TagRaumListe, ZeitListe, Stream),
	schreibeTerminNachAbspracheXML(Stream),
	close(Stream).

/***********************************************************/
/* schreibePlanXML1(+TagListe, +RaumListe, +TagRaumListe,  */
/*     +ZeitListe, ?Stream)                                */
/***********************************************************/
/* Schreiben der Vorlesungstage und des jeweils ersten     */
/* Raumes                                                  */
/***********************************************************/
schreibePlanXML1([], _, _, _, _).
schreibePlanXML1([TagKopf|TagListe], [RaumKopf|RaumListe], TagRaumListe, [ZeitKopf|ZeitRest], Stream) :-
	schreibeWochentagXML(TagKopf, Stream),
	schreibeRaumXML(RaumKopf, Stream),
	schreibePlanXML3([TagKopf|TagListe], [RaumKopf|RaumListe], TagRaumListe, [ZeitKopf|ZeitRest], Stream).

/***********************************************************/
/* schreibePlanXML2(+TagListe, +RaumListe, +TagRaumListe,  */
/*     +ZeitListe, ?Stream)                                */
/***********************************************************/
/* Schreiben der Raeume (ausser 1. Raum)                   */
/***********************************************************/
schreibePlanXML2(TagListe, [RaumKopf|RaumListe], TagRaumListe, ZeitListe, Stream) :-
        schreibeRaumXML(RaumKopf, Stream),
        schreibePlanXML3(TagListe, [RaumKopf|RaumListe], TagRaumListe, ZeitListe, Stream).

/***********************************************************/
/* schreibePlanXML3(+TagListe, +RaumListe, +TagRaumListe,  */
/*     +ZeitListe, ?Stream)                                */
/***********************************************************/
/* Schreiben der Vorlesungen                               */
/*    /1 - letzte Vorlesung eines Tages (Verwaltung)       */
/*    /2 - letzte Vorlesung eines Raumes                   */
/*    /3 - ohne Einschraenkungen                           */
/***********************************************************/
schreibePlanXML3([_|TagListe], [_], TagRaumListe, [], Stream) :-
	schliesseRaumXML(Stream),
	schliesseWochentagXML(Stream),
        erstelleRaumListe(RaumListe),
        erstelleZeitListe(ZeitListe),
        schreibePlanXML1(TagListe, RaumListe, TagRaumListe, ZeitListe, Stream).
schreibePlanXML3(TagListe, [_|RaumListe], TagRaumListe, [], Stream) :-
	schliesseRaumXML(Stream),
        erstelleZeitListe(ZeitListe),
        schreibePlanXML2(TagListe, RaumListe, TagRaumListe, ZeitListe, Stream).
schreibePlanXML3(TagListe, RaumListe, [TagRaumKopf|TagRaumListe], [ZeitKopf|ZeitListe], Stream) :-
	schreibeZeitXML(ZeitKopf, Stream),
        schreibeVeranstaltungXML(TagRaumKopf, Stream),
	schliesseZeitXML(Stream),
        schreibePlanXML3(TagListe, RaumListe, TagRaumListe, ZeitListe, Stream).

/***********************************************************/
/* schreibeTerminNachAbspracheXML(?Stream)                 */
/***********************************************************/
/* Schreiben der "Termine nach Absprache",                 */
/* Steuerung des Ablaufes.                                 */
/***********************************************************/
schreibeTerminNachAbspracheXML(Stream):-
	findall(Name, terminNachAbsprache(Name, _), Liste),
	schreibeTerminNachAbspracheXML(Liste, Stream).

/***********************************************************/
/* schreibeTerminNachAbspracheXML(+Liste, ?Stream)         */
/***********************************************************/
/* Schreiben der terminNachAbsprache-Daten.                */
/***********************************************************/
schreibeTerminNachAbspracheXML([], _).
schreibeTerminNachAbspracheXML([Kopf|Rest], Stream) :-
	print(Stream, '      <absprache>'),
	nl(Stream),
	schreibeVeranstaltungTNA(Kopf, Stream),
	print(Stream, '      </absprache>'),
	nl(Stream),
	schreibeTerminNachAbspracheXML(Rest, Stream).

/***********************************************************/
/* schreibeVeranstaltungTNA(+Name, ?Stream)                */
/***********************************************************/
/* Schreibe eine Veranstaltung fuer einen Termin nach      */
/* Absprache.  Hierbei ist optisch eine andere Einrueckung */
/* notwendig.                                              */
/***********************************************************/
schreibeVeranstaltungTNA(Name, Stream) :-
	print(Stream, '            <veranstaltung name=\"'),
	print(Stream, Name),
	print(Stream, '\">'),
	nl(Stream),
	findall(Teilnehmer, terminNachAbsprache(Name, Teilnehmer), TeilnehmerListe),
	flatten(TeilnehmerListe, ListeFlat),
	schreibeTeilnehmerTNA(ListeFlat, Stream),
	print(Stream, '            </veranstaltung>'),
	nl(Stream).
	
/***********************************************************/
/* schreibeTeilnehmerTNA(+Teilnehmer, ?Stream)             */
/***********************************************************/
/* Schreiben eines Teilnehmers mit anderer Einrueckung     */
/***********************************************************/
schreibeTeilnehmerTNA([], _).
schreibeTeilnehmerTNA([Kopf|Rest], Stream) :-
	dozenten(Dozenten),
	member(Kopf, Dozenten),
	print(Stream, '                  <teilnehmer name =\"'),
	print(Stream, Kopf),
	print(Stream, '\" dozent=\"true\" />'),
	nl(Stream),
	schreibeTeilnehmerTNA(Rest, Stream).
schreibeTeilnehmerTNA([Kopf|Rest], Stream) :-
	print(Stream, '                  <teilnehmer name =\"'),
	print(Stream, Kopf),
	print(Stream, '\"/>'),
	nl(Stream),
	schreibeTeilnehmerTNA(Rest, Stream).

/***********************************************************/
/* schreibeWochentag(+Tag, +Raum, ?Stream)                 */
/***********************************************************/
/* Schreiben eines Wochentages                             */
/***********************************************************/
schreibeWochentagXML(Tag, Stream) :-
	print(Stream, '      <tag name=\"'),
	print(Stream, Tag),
	print(Stream, '\">'),
        nl(Stream).

/***********************************************************/
/* schliesseWochentag(?Stream)                             */
/***********************************************************/
/* Schreiben eines Wochentag-Tags                          */
/***********************************************************/
schliesseWochentagXML(Stream) :-
	print(Stream, '      </tag>'),
	nl(Stream).

/***********************************************************/
/* schreibeRaum(+Raum, ?Stream)                            */
/***********************************************************/
/* Schreiben eines Raumes                                  */
/***********************************************************/
schreibeRaumXML(Raum, Stream) :-
        print(Stream, '            <raum name=\"'),
        print(Stream, Raum),
        print(Stream, '\">'),
        nl(Stream).

/***********************************************************/
/* schliesseRaum(?Stream)                                  */
/***********************************************************/
/* Schliessen des Raum-Tags                                */
/***********************************************************/
schliesseRaumXML(Stream) :-
	print(Stream, '            </raum>'),
	nl(Stream).
	
/***********************************************************/
/* schreibeZeit(+Zeit, ?Stream)                            */
/***********************************************************/
/* Schreiben eines Zeit                                    */
/***********************************************************/
schreibeZeitXML(Zeit, Stream) :-
	print(Stream, '                  <zeit name=\"'),
        print(Stream, Zeit),
        print(Stream, '\">'),
        nl(Stream).

/***********************************************************/
/* schliesseZeit(?Stream)                                  */
/***********************************************************/
/* Schliessen des Zeit-Tags                                */
/***********************************************************/
schliesseZeitXML(Stream) :-
	print(Stream, '                  </zeit>'),
	nl(Stream).

/***********************************************************/
/* schreibeVeranstaltung(+ListenElement, ?Stream)          */
/***********************************************************/
/* Schreiben einer Veranstaltung                           */
/***********************************************************/
schreibeVeranstaltungXML([], Stream) :-
	print(Stream, '                        <!-- keine Veranstaltung -->'),
        nl(Stream).
schreibeVeranstaltungXML([(Veranstaltung, Teilnehmer, _, _, _)], Stream) :-
	print(Stream, '                        <veranstaltung name=\"'),
	print(Stream, Veranstaltung),
	print(Stream, '\">'),
	nl(Stream),
	schreibeTeilnehmerXML(Teilnehmer, Stream),
	print(Stream, '                        </veranstaltung>'),
	nl(Stream).

/***********************************************************/
/* schreibeTeilnehmerXML(+Teilnehmer, ?Stream)             */
/***********************************************************/
/* Schreiben eines Teilnehmers                             */
/***********************************************************/
schreibeTeilnehmerXML([], _).
schreibeTeilnehmerXML([Kopf|Rest], Stream) :-
	dozenten(Dozenten),
	member(Kopf, Dozenten),
	print(Stream, '                              <teilnehmer name =\"'),
	print(Stream, Kopf),
	print(Stream, '\" dozent=\"true\" />'),
	nl(Stream),
	schreibeTeilnehmerXML(Rest, Stream).
schreibeTeilnehmerXML([Kopf|Rest], Stream) :-
	print(Stream, '                              <teilnehmer name =\"'),
	print(Stream, Kopf),
	print(Stream, '\"/>'),
	nl(Stream),
	schreibeTeilnehmerXML(Rest, Stream).

/***********************************************************/
/* ersetzeUntergruppen(+Liste, +VergleichsListe, -Ergebnis)*/
/***********************************************************/
/* Ersetzen der Untergruppen in den Veranstlatungen, so    */
/* dass diese nicht mehr im fertigen Plan auftauchen.      */
/***********************************************************/
ersetzeUntergruppen([], _, []).
ersetzeUntergruppen([veranstaltung(VerName, VerTeilnehmer, VerRessourcen)|Rest], UGruppenListe, [ErgebnisKopf|ErgebnisRest]) :-
	findeUntergruppen(VerTeilnehmer, UGruppenListe, NeueTeilnehmer),
	flatten(NeueTeilnehmer, TeilnehmerFlat),
	append([veranstaltung(VerName, TeilnehmerFlat, VerRessourcen)], [], ErgebnisKopf),
	ersetzeUntergruppen(Rest, UGruppenListe, ErgebnisRest).

/***********************************************************/
/* findeUntergruppen(+Liste, +VergleichsListe, -Ergebnis)  */
/***********************************************************/
/* Auffinden der Untergruppen und ersetzen dieser durch    */
/* die zugehoerige Teilnehmergruppe.                       */
/***********************************************************/
findeUntergruppen([], _, []).
findeUntergruppen([Kopf|Rest], UGruppenListe, [ErgebnisKopf|ErgebnisRest]) :-
	member((Kopf, Wert), UGruppenListe),
	append([Wert], [], ErgebnisKopf),
	findeUntergruppen(Rest, UGruppenListe, ErgebnisRest).
findeUntergruppen([Kopf|Rest], UGruppenListe, [ErgebnisKopf|ErgebnisRest]) :-
	nonmember((Kopf, _), UGruppenListe),
	append([Kopf], [], ErgebnisKopf),
	findeUntergruppen(Rest, UGruppenListe, ErgebnisRest).

/***********************************************************/
/* schreibeVeranstListeOhneUntergruppen(+VeranstListe,     */
/*	?Ausgabedatei)                                     */
/***********************************************************/
/* Schreiben der veranstaltung(_, _, _)-Klauseln in die    */
/* Ausgabedatei                                            */
/***********************************************************/
schreibeVeranstListeOhneUntergruppen(VeranstaltungsListe, Datei) :-
	open(Datei, write, Stream),
	schreibeVeranstListeOhneUntergruppen1(VeranstaltungsListe, Stream),
	print(Stream, '% Veranstaltungen ohne Untergruppen'),
	nl(Stream),	
	close(Stream).

/***********************************************************/
/* schreibeVeranstListeOhneUntergruppen1(+VeranstListe,    */
/*	?Ausgabedatei)                                     */
/***********************************************************/
/* Schreiben der veranstaltung(_, _, _)-Klauseln in die    */
/* Ausgabedatei                                            */
/***********************************************************/
schreibeVeranstListeOhneUntergruppen1([], Stream) :-
	nl(Stream).
schreibeVeranstListeOhneUntergruppen1([veranstaltung(VerName, VerTeilnehmer, VerRessourcen)|Rest], Stream) :-
	print(Stream, 'veranstaltung('),
	print(Stream, '\''),
	print(Stream, VerName),
	print(Stream, '\', '),
	print(Stream, VerTeilnehmer),
	print(Stream, ', '),
	print(Stream, VerRessourcen),
	print(Stream, ').'),
	nl(Stream),   
	schreibeVeranstListeOhneUntergruppen1(Rest, Stream).
