%
% harteConstrains.pl
%
% Berechnung der harten Constraints
%

/***********************************************************/
/* berechneEssentieleEinschraenkungen(?Array)              */
/***********************************************************/
/* Berechnen der Vorlesungen im Stundenplan                */
/***********************************************************/
berechneEssentielleEinschraenkungen(Stundenplan) :-
	anzVeranst(Index),
	findall(RaumNo, raum(RaumNo, _, _, _, _), RaumListe),
	writeln("    Einschrnken der Rume ..."),
	einschraenkenDerRaeume(Stundenplan, Index, RaumListe),
	writeln("    Verhindern von Raumkollisionen ..."),
	verhindereRaumkollisionen(Stundenplan, Index),
	writeln("    Verhindern von Teilnehmerkollisionen ..."),
	verhindereTeilnehmerkollisionen(Stundenplan).

/***********************************************************/
/* einschraenkenDerRaeume(?Array, +Index, +RaumListe)      */
/***********************************************************/
/* Einschraenken der Raeume nach Ressourcen und            */
/* Teilnehmeranzahl                                        */
/***********************************************************/
einschraenkenDerRaeume(_, 0, _).
einschraenkenDerRaeume(Stundenplan, IndexAlt, RaumListe) :-
	subscript(Stundenplan, [IndexAlt, 1], Raum),
	veranstaltung(IndexAlt, _, _, VerRessourcen, AnzahlTeilnehmer),
        % Auswertung des oder(_)-Praedikats -> kein oder(_)
	nonmember(oder(_), VerRessourcen),
	bestimmeGueltigeRaeume(RaumListe, VerRessourcen, AnzahlTeilnehmer, GueltigeRaeume),
	Raum :: GueltigeRaeume,
	Index is IndexAlt - 1,
	einschraenkenDerRaeume(Stundenplan, Index, RaumListe).
einschraenkenDerRaeume(Stundenplan, IndexAlt, RaumListe) :-
	subscript(Stundenplan, [IndexAlt, 1], Raum),
	veranstaltung(IndexAlt, _, _, VerRessourcen, AnzahlTeilnehmer),
        % Auswertung des oder(_)-Praedikats -> oder(_) vorhanden
	(member(oder(Elemente), VerRessourcen), 
	erstelleNeueRessourcenListe(Elemente, VerRessourcen, NeueVerRessourcenListe)),
	bereiteBestimmeGueltigeRaeumeVor(RaumListe, NeueVerRessourcenListe, AnzahlTeilnehmer, GueltigeRaeume),
	flatten(GueltigeRaeume, GueltigeRaeumeFlat),
	Raum :: GueltigeRaeumeFlat,
	Index is IndexAlt - 1,
	einschraenkenDerRaeume(Stundenplan, Index, RaumListe).

bereiteBestimmeGueltigeRaeumeVor(_, [], _, []).
bereiteBestimmeGueltigeRaeumeVor(RaumListe, [VerRessourcenKopf|VerRessourcenRest], AnzahlTeilnehmer, [GueltigeRaeumeKopf|GueltigeRaeumeRest]) :-
	bestimmeGueltigeRaeume(RaumListe, VerRessourcenKopf, AnzahlTeilnehmer, GueltigeRaeumeKopf),
	bereiteBestimmeGueltigeRaeumeVor(RaumListe, VerRessourcenRest, AnzahlTeilnehmer, GueltigeRaeumeRest).

/***********************************************************/
/* bestimmeGueltigeRaeume(+RaumListe, +ID,                 */
/*     -GueltigeRaeume)                                    */
/***********************************************************/
bestimmeGueltigeRaeume([], _, _, []).
bestimmeGueltigeRaeume([RaumKopf|RaumRest], VerRessourcen, AnzahlTeilnehmer, GueltigeRaeume) :-
	raum(RaumKopf, _, Groesse, _, _),
	Groesse < AnzahlTeilnehmer,
	bestimmeGueltigeRaeume(RaumRest, VerRessourcen, AnzahlTeilnehmer, GueltigeRaeume).
bestimmeGueltigeRaeume([RaumKopf|RaumRest], VerRessourcen, AnzahlTeilnehmer, GueltigeRaeume) :-
	raum(RaumKopf, _, Groesse, RaumRessourcen, _),
	Groesse >= AnzahlTeilnehmer,
	not(istTeilmenge(VerRessourcen, RaumRessourcen)),
	bestimmeGueltigeRaeume(RaumRest, VerRessourcen, AnzahlTeilnehmer, GueltigeRaeume).
bestimmeGueltigeRaeume([RaumKopf|RaumRest], VerRessourcen, AnzahlTeilnehmer, [GueltigerRaum|GueltigeRaeume]) :-
%	veranstaltung(VerNo, _, _, VerRessourcen, AnzahlTeilnehmer),
	raum(RaumKopf, _, Groesse, RaumRessourcen, _),
	Groesse >= AnzahlTeilnehmer,
	istTeilmenge(VerRessourcen, RaumRessourcen),
	GueltigerRaum is RaumKopf,
	bestimmeGueltigeRaeume(RaumRest, VerRessourcen, AnzahlTeilnehmer, GueltigeRaeume).

erstelleNeueRessourcenListe([], _, []).
erstelleNeueRessourcenListe([ElementKopf|ElementRest], VerRessourcen, [NeueVerRessourcenKopf|NeueVerRessourcenListe]) :-
	delete(oder(_), VerRessourcen, VerRessourcenNeu),
	append([ElementKopf], VerRessourcenNeu, NeueVerRessourcenKopf),
	erstelleNeueRessourcenListe(ElementRest, VerRessourcen, NeueVerRessourcenListe).

/***********************************************************/
/* verhindereRaumkollisionen(?Array, +Index)               */
/***********************************************************/
/* Verhindern von Doppelbelegungen zweier Raeume           */
/***********************************************************/
verhindereRaumkollisionen(Stundenplan, Index) :-
	anzTage(AnzTage),
	anzZeiten(AnzZeiten),
	sorgeFuerUnterschiedlicheRaeume(Stundenplan, Index, Liste, AnzTage, AnzZeiten),
	alldifferent(Liste).

/***********************************************************/
/* sorgeFuerUnterschiedlicheRaeume(?Array, +Index,         */
/*     -Ergebnis, -AnzTage, -AnzZeiten)                    */
/***********************************************************/
/* Durchsetzen der unterschiedlichen Raeume mit Hilfe      */
/* von Delay                                               */
/***********************************************************/
sorgeFuerUnterschiedlicheRaeume(_, 0, _, _, _).
sorgeFuerUnterschiedlicheRaeume(Stundenplan, IndexAlt, [Kopf|Liste], AnzTage, AnzZeiten) :-
	subscript(Stundenplan, [IndexAlt, 1], Raum),
	subscript(Stundenplan, [IndexAlt, 2], Tag),
	subscript(Stundenplan, [IndexAlt, 3], Zeit),
	AnzTageUndZeiten is AnzTage * AnzZeiten,
	(AnzTageUndZeiten * Raum + AnzZeiten * Tag + Zeit) #= Kopf,
	Index is IndexAlt - 1,
	sorgeFuerUnterschiedlicheRaeume(Stundenplan, Index, Liste, AnzTage, AnzZeiten).

/***********************************************************/
/* verhindereTeilnehmerkollisionen(?Array)                 */
/***********************************************************/
/* Verhindern von Doppelbuchungen zweier Teilnehmer        */
/***********************************************************/
verhindereTeilnehmerkollisionen(Stundenplan) :-
	anzZeiten(AnzZeiten),
	findall(Veranstaltungen, gleicheTeilnehmer(Veranstaltungen), GleicheTeilnehmerListe),
	verhindereTeilnehmerkollisionen(Stundenplan, GleicheTeilnehmerListe, AnzZeiten).

/***********************************************************/
/* verhindereTeilnehmerkollisionen(?Array,                 */
/***********************************************************/
/*     +GlTeilnehmerListe, +AnzZeiten)                     */
/***********************************************************/
verhindereTeilnehmerkollisionen(_, [], _).
verhindereTeilnehmerkollisionen(Stundenplan, [GLKopf|RestListe], AnzZeiten) :-
	sorgeFuerUnterschiedlicheZeiten(Stundenplan, GLKopf, AnzZeiten, UntZeitenListe),
	alldifferent(UntZeitenListe),
	verhindereTeilnehmerkollisionen(Stundenplan, RestListe, AnzZeiten).

/***********************************************************/
/* sorgeFuerUnterschiedlicheZeiten(?Array,                 */
/*     +GlTeilnehmerListe, +AnzZeiten, -Ergebnis)          */
/***********************************************************/
/* Durchsetzen der unterschiedlichen Teilnehmern mit Hilfe */
/* von Delay                                               */
/***********************************************************/
sorgeFuerUnterschiedlicheZeiten(_, [], _, []).
sorgeFuerUnterschiedlicheZeiten(Stundenplan, [IndexKopf|IndexRest], AnzZeiten, [Kopf|Rest]) :- 
	subscript(Stundenplan, [IndexKopf, 2], Tag),
	subscript(Stundenplan, [IndexKopf, 3], Zeit),
	(AnzZeiten * Tag + Zeit) #= Kopf,
	sorgeFuerUnterschiedlicheZeiten(Stundenplan, IndexRest, AnzZeiten,Rest).
