%
% harteConstrains.pl
%
% Berechnung der harten Constraints
%

% letzte Aenderung 7.11.01 mb: Fakten liegen bereits als Nummern vor

% letzte Aenderung 16.11.01 mb: Fakt 'zurGleichenZeit' hinzugefuegt

/***********************************************************/
/* berechneHarteConstraints(?Array)                        */
/***********************************************************/
/* Berechnen der harten Constraints im Stundenplan         */
/***********************************************************/
berechneHarteConstraints(Stundenplan) :-

    % veranstaltungAmUmIn auswerten
    writeln("   Einschrnkungen bezglich der Veranstaltungen auswerten ... "), 
    findall((VerID, TagIDs, ZeitIDs), veranstaltungAmUmInV(VerID, TagIDs, ZeitIDs, _), AmUmListeFinal),

    %veranstaltungNichtAmUmIn auswerten
    findall((VerID, TagIDs, ZeitIDs), veranstaltungNichtAmUmInV(VerID, TagIDs, ZeitIDs, _), NichtAmUmListeFinal),

    % veranstaltungenAmSelbenTag auswerten
    findall((VerID1, VerID2), veranstaltungenAmSelbenTagV(VerID1, VerID2), AmSelbenTagListe),

    % veranstaltungenNichtAmSelbenTag auswerten
    findall((VerID1, VerID2), veranstaltungenNichtAmSelbenTagV(VerID1, VerID2), NichtAmSelbenTagListe),

    % veranstaltungNichtZusammenhaengend
    findall((VerID1, VerID2), veranstaltungenNichtZusammenhaengendV(VerID1, VerID2), NichtZusammenhaengendListe),

    % doppelstunde usw. auswerten
    % hier werden auch dreifach- und vierfachstunden beruecksichtigt
    writeln("   Doppelstunden auswerten ... "), 
    findall((VerID1, VerID2), doppelstundeV(VerID1, VerID2), DoppelstundenListe), 

    % zurGleichenZeit auswerten
    writeln("   Veranstaltungen, die zur gleichen Zeit stattfinden, auswerten ..."),
    findall((VerID1, VerID2), zurGleichenZeitV(VerID1, VerID2), ZurGleichenZeitListe),

    anzZeiten(AnzZeiten),
    writeln("   Einschrnkungen bertragen ..."),

    uebertrageAufStundenplan1(Stundenplan, AmUmListeFinal),
    uebertrageAufStundenplan2(Stundenplan, NichtAmUmListeFinal, AnzZeiten),
    uebertrageAufStundenplan3(Stundenplan, AmSelbenTagListe),
    uebertrageAufStundenplan4(Stundenplan, NichtAmSelbenTagListe),
    uebertrageAufStundenplan5(Stundenplan, NichtZusammenhaengendListe),
    uebertrageAufStundenplan6(Stundenplan, DoppelstundenListe),
    uebertrageAufStundenplan7(Stundenplan, ZurGleichenZeitListe).

/***********************************************************/
/* uebertrageAufStundenplan(Array, -Ergebnis)             */
/***********************************************************/
/* uebertragen der gefundenen harten Einschraenkungen in   */
/* das StundenplanArray                                    */
/* 1: fuer AmUmEinschraenkungen                            */
/* 2: fuer NichtAmUmEinschraenkungen                       */
/* 3: fuer AmSelbenTagEinschraenkungen                     */
/* 4: fuer NichtAmSelbenTagEinschraenkungen                */
/* 5: fuer NichtZusammenhaengendEinschraenkungen           */
/* 6: fuer Doppelstunden                                   */
/* 7: fuer zurGleichenZeit                                 */
/***********************************************************/
uebertrageAufStundenplan1(_, []).
uebertrageAufStundenplan1(Stundenplan, [(ID, Tage, [])|Rest]) :-
    subscript(Stundenplan, [ID, 2], Tag),
    Tag :: Tage,
    uebertrageAufStundenplan1(Stundenplan, Rest).
uebertrageAufStundenplan1(Stundenplan, [(ID, [], Zeiten)|Rest]) :-
    subscript(Stundenplan, [ID, 3], Zeit),
    Zeit :: Zeiten,
    uebertrageAufStundenplan1(Stundenplan, Rest).
uebertrageAufStundenplan1(Stundenplan, [(ID, Tage, Zeiten)|Rest]) :-
    subscript(Stundenplan, [ID, 2], Tag),
    subscript(Stundenplan, [ID, 3], Zeit),
    Tag :: Tage,
    Zeit :: Zeiten,
    uebertrageAufStundenplan1(Stundenplan, Rest).
uebertrageAufStundenplan2(_, [], _).
uebertrageAufStundenplan2(Stundenplan, [(ID, [], Zeiten)|Rest], AnzZeiten) :-
    findall(TagNo, tag(TagNo, _, _), TagListe), 
    subscript(Stundenplan, [ID, 2], Tag),
    subscript(Stundenplan, [ID, 3], Zeit), 
    berechneEinschraenkungen(Tag, Zeit, AnzZeiten, TagListe, Zeiten),
    uebertrageAufStundenplan2(Stundenplan, Rest, AnzZeiten).
uebertrageAufStundenplan2(Stundenplan, [(ID, Tage, [])|Rest], AnzZeiten) :-
    findall(ZeitNo, zeit(ZeitNo, _, _), ZeitListe),
    subscript(Stundenplan, [ID, 2], Tag),
    subscript(Stundenplan, [ID, 3], Zeit), 
    berechneEinschraenkungen(Tag, Zeit, AnzZeiten, Tage, ZeitListe),
    uebertrageAufStundenplan2(Stundenplan, Rest, AnzZeiten).
uebertrageAufStundenplan2(Stundenplan, [(ID, Tage, Zeiten)|Rest], AnzZeiten) 
:-
    subscript(Stundenplan, [ID, 2], Tag),
    subscript(Stundenplan, [ID, 3], Zeit), 
    berechneEinschraenkungen(Tag, Zeit, AnzZeiten, Tage, Zeiten),
    uebertrageAufStundenplan2(Stundenplan, Rest, AnzZeiten).
uebertrageAufStundenplan3(_, []).
uebertrageAufStundenplan3(Stundenplan, [(ID1, ID2)|Rest]) :-
    subscript(Stundenplan, [ID1, 2], Tagi),
    subscript(Stundenplan, [ID2, 2], Tagj),
    Tagi #= Tagj,
    uebertrageAufStundenplan3(Stundenplan, Rest).
uebertrageAufStundenplan4(_, []).
uebertrageAufStundenplan4(Stundenplan, [(ID1, ID2)|Rest]) :-
    subscript(Stundenplan, [ID1, 2], Tagi),
    subscript(Stundenplan, [ID2, 2], Tagj),
    Tagi #\= Tagj,
    uebertrageAufStundenplan4(Stundenplan, Rest).
uebertrageAufStundenplan5(_, []).
uebertrageAufStundenplan5(Stundenplan, [(ID1, ID2)|Rest]) :-
    subscript(Stundenplan, [ID1, 3], Zeiti),
    subscript(Stundenplan, [ID2, 3], Zeitj),
    Zeiti #\= (Zeitj + 1), 
    (Zeiti + 1) #\= Zeitj, 
    uebertrageAufStundenplan5(Stundenplan, Rest).
uebertrageAufStundenplan6(_, []).
uebertrageAufStundenplan6(Stundenplan, [(ID1, ID2)|Rest]) :-
    subscript(Stundenplan, [ID1, 1], Raumi),
    subscript(Stundenplan, [ID2, 1], Raumj),
    subscript(Stundenplan, [ID1, 2], Tagi),
    subscript(Stundenplan, [ID2, 2], Tagj),
    subscript(Stundenplan, [ID1, 3], Zeiti),
    subscript(Stundenplan, [ID2, 3], Zeitj),
    Raumj #= Raumi,
    Tagj #= Tagi,
    Zeitj #= (Zeiti + 1), 
    uebertrageAufStundenplan6(Stundenplan, Rest).
uebertrageAufStundenplan7(_, []).
uebertrageAufStundenplan7(Stundenplan, [(ID1, ID2)|Rest]) :-
    subscript(Stundenplan, [ID1, 2], Tagi),
    subscript(Stundenplan, [ID2, 2], Tagj),
    subscript(Stundenplan, [ID1, 3], Zeiti),
    subscript(Stundenplan, [ID2, 3], Zeitj),
    Tagj #= Tagi,
    Zeitj #= Zeiti, 
    uebertrageAufStundenplan7(Stundenplan, Rest).

/***********************************************************/
/* berechneEinschraenkungen(+Tag, +Zeit, +AnzZeiten,       */
/*	+Tage, +Zeiten)                                    */
/***********************************************************/
/* Festlegen der Einschraenkungen mittels Delay            */
/***********************************************************/
berechneEinschraenkungen(_, _, _, _, []).
berechneEinschraenkungen(_, _, _, [], _).
berechneEinschraenkungen(_, _, _, [], []).
berechneEinschraenkungen(Tag, Zeit, AnzZeiten, [TagKopf|TagRest], [ZeitKopf|ZeitRest]) :-
    (AnzZeiten * Tag + Zeit) #\= (AnzZeiten * TagKopf + ZeitKopf),
    berechneEinschraenkungen(Tag, Zeit, AnzZeiten, [TagKopf], ZeitRest),
    berechneEinschraenkungen(Tag, Zeit, AnzZeiten, TagRest, [ZeitKopf|ZeitRest]).    
